
package kz.project.printedFormsService.config;

import kz.project.printedFormsService.controller.DTemplateTypeController;
import kz.project.printedFormsService.controller.DocumentsController;
import kz.project.printedFormsService.controller.GitlabTemplateController;
import kz.project.printedFormsService.controller.TemplateController;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.convert.converter.Converter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

import java.util.Arrays;
import java.util.List;

import static kz.project.printedFormsService.controller.TemplateController.BASE_PATH;

@Configuration
@EnableWebSecurity
@EnableMethodSecurity
public class SecurityConfiguration {

    private static final String CREATOR = "creator";
    private static final String DELETE = "delete";
    private static final String EDITOR = "editor";
    private static final String ADMIN = "admin";

    @Bean
    public SecurityFilterChain configure(HttpSecurity http,
                                         Converter<Jwt, ? extends AbstractAuthenticationToken> jwtAuthenticationConverter) throws Exception {

        http.oauth2ResourceServer(oauth2 -> oauth2.jwt(jwt -> jwt.jwtAuthenticationConverter(jwtAuthenticationConverter)));

        http.cors(cors -> cors.configurationSource(corsConfigurationSource("*")));

        http.sessionManagement(sm -> sm.sessionCreationPolicy(SessionCreationPolicy.STATELESS));

        http.csrf(AbstractHttpConfigurer::disable);

        http.exceptionHandling(eh -> eh.authenticationEntryPoint((request, response, authException) -> {
            response.addHeader(HttpHeaders.WWW_AUTHENTICATE, "Bearer realm=\"Restricted Content\"");
            response.sendError(HttpStatus.UNAUTHORIZED.value(), HttpStatus.UNAUTHORIZED.getReasonPhrase());
        }));

        http.authorizeHttpRequests(accessManagement -> accessManagement
                .requestMatchers("/actuator/health/readiness",
                        "/actuator/health/liveness",
                        "/api-docs/**",
                        "/swagger-ui/**",
                        BASE_PATH + TemplateController.GET_TEMPLATE,
                        DTemplateTypeController.BASE_PATH+DTemplateTypeController.GET_ALL
                )
                .permitAll()

                .requestMatchers(BASE_PATH + TemplateController.SAVE)
                .hasAnyRole(CREATOR, ADMIN)

                .requestMatchers(BASE_PATH + TemplateController.GET_TEMPLATE_DATA)
                .hasAnyRole(CREATOR, EDITOR, DELETE, ADMIN)

                .requestMatchers(BASE_PATH + TemplateController.EDIT)
                .hasAnyRole(EDITOR, ADMIN)

                .requestMatchers(BASE_PATH + TemplateController.DELETE)
                .hasAnyRole(DELETE, ADMIN)

                .requestMatchers(BASE_PATH + TemplateController.GET_ALL)
                .hasAnyRole(CREATOR, EDITOR, DELETE, ADMIN)

                .requestMatchers(BASE_PATH + TemplateController.GET_ALL_BY_CODE)
                .hasAnyRole(CREATOR, EDITOR, DELETE, ADMIN)

                .requestMatchers(DocumentsController.BY_DAY)
                .hasAnyRole(CREATOR, EDITOR, DELETE, ADMIN)

                .requestMatchers(DocumentsController.BY_TEMPLATE)
                .hasAnyRole(CREATOR, EDITOR, DELETE, ADMIN)

                .requestMatchers(GitlabTemplateController.SAVE)
                .hasAnyRole(CREATOR, ADMIN)

                .requestMatchers(GitlabTemplateController.EDIT)
                .hasAnyRole(EDITOR, ADMIN)

                .anyRequest().authenticated()
        );


        return http.build();
    }

    private UrlBasedCorsConfigurationSource corsConfigurationSource(String... origins) {
        final var configuration = new CorsConfiguration();
        configuration.setAllowedOrigins(Arrays.asList(origins));
        configuration.setAllowedMethods(List.of("*"));
        configuration.setAllowedHeaders(List.of("*"));
        configuration.setExposedHeaders(List.of("*"));

        final var source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);
        return source;
    }
}
