package kz.project.printedFormsService.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import kz.project.printedFormsService.data.dto.DTemplateTypeDto;
import kz.project.printedFormsService.logging.ProcessLogger;
import kz.project.printedFormsService.service.DTemplateTypeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static kz.project.printedFormsService.controller.DTemplateTypeController.BASE_PATH;

@RestController
@RequestMapping(BASE_PATH)
@Slf4j
@Tag(name = "DTemplate Controller", description = "API TemplateService")
public class DTemplateTypeController {
  private static final Logger LOGGER = LoggerFactory.getLogger(DTemplateTypeController.class);
  public final static String BASE_PATH = "/api/dict";
  public final static String GET_ALL = "/get/all";

  private final DTemplateTypeService service;
  private ProcessLogger processLogger;

  public DTemplateTypeController(DTemplateTypeService service, ProcessLogger processLogger) {
    this.service = service;
    this.processLogger = processLogger;
  }

  @GetMapping("get/all")
    @Operation(description = "Метод для получения шаблона по идентификатору")
    public ResponseEntity getDict() {
        try {
          List<DTemplateTypeDto> dtoList = service.getAllTemplateType();
          return ResponseEntity.ok(dtoList);
        } catch (Exception e){
          LOGGER.error(e.getMessage(), e);
          processLogger.error(TemplateController.class, new Throwable().getStackTrace()[0].getMethodName(), "Получение типов шаблонов");
          return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(e.getMessage());
        }
    }
}
