package kz.project.printedFormsService.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import kz.project.printedFormsService.data.dto.GitlabUploadRequest;
import kz.project.printedFormsService.data.dto.TemplateDto;
import kz.project.printedFormsService.data.dto.TemplateResponseDto;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.service.GitlabTemplateService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;

@RestController
@RequiredArgsConstructor
@Slf4j
@Tag(name = "Gitlab Template Controller", description = "API GitlabTemplateService")
public class GitlabTemplateController {

    public static final String BASE_PATH = "/api/gitlabTemplate";
    public static final String SAVE = BASE_PATH + "/save";
    public static final String EDIT = BASE_PATH + "/edit";

    private final GitlabTemplateService gitlabTemplateService;


    @PostMapping(SAVE)
    @Operation(description = "Метод сохранения шаблона")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = TemplateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })
    public ResponseEntity<TemplateDto> saveTemplate(
            @RequestBody @Validated GitlabUploadRequest request
    ) throws IOException, ValidationException {
        return ResponseEntity.ok(gitlabTemplateService.saveFromGitlab(request));

    }


    @PutMapping(EDIT)
    @Operation(description = "Метод редактирования шаблона")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Данные получены успешно",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = TemplateDto.class))
                    }),


            @ApiResponse(
                    responseCode = "400",
                    description = "Ошибка сервиса",
                    content = {
                            @Content(
                                    mediaType = "application/json",

                                    schema = @Schema(implementation = ValidationException.class))
                    })
    })
    public ResponseEntity<TemplateResponseDto> editTemplate(
            @RequestBody @Validated GitlabUploadRequest request
    ) throws IOException, ValidationException {
        return ResponseEntity.ok(gitlabTemplateService.updateFromGitlab(request));

    }
}
