package kz.project.printedFormsService.converter;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.jsonpath.JsonPath;
import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.convert.converter.Converter;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationToken;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class JwtAuthConverter implements Converter<Jwt, AbstractAuthenticationToken> {
    public static Map<String, Object> getTokenClaims(String token) throws ParseException {
        JWT parsedJwt = JWTParser.parse(token);
        return parsedJwt.getJWTClaimsSet().getClaims();
    }

    public static String getUsername(String token) throws ParseException {
        Map<String, Object> claims = getTokenClaims(token);
        if (claims == null) return null;
        return (String) claims.get("preferred_username");
    }

    @Override
    public AbstractAuthenticationToken convert(Jwt jwt) {
        Collection<GrantedAuthority> authorities = extractAuthorities(jwt);
        final String username = JsonPath.read(jwt.getClaims(), "preferred_username");

        return new JwtAuthenticationToken(jwt, authorities, username);
    }

    private Collection<GrantedAuthority> extractAuthorities(Jwt jwt) {
        if (jwt.getClaim("resource_access") != null) {
            Map<String, Map<String,Object>> resourceAccess = jwt.getClaim("resource_access");
            if (resourceAccess.containsKey("print_form")){
                ObjectMapper mapper = new ObjectMapper();
                List<String> roles = mapper.convertValue(resourceAccess.get("print_form").get("roles"), new TypeReference<>() {
                });
                Set<GrantedAuthority> authorities = new HashSet<>();

                for (String role : roles) {
                    authorities.add(new SimpleGrantedAuthority("ROLE_" + role.substring(role.lastIndexOf('_')+1)));
                }

                return authorities;
            }

        }
        return new ArrayList<>();
    }
}
