package kz.project.printedFormsService.data.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import kz.project.printedFormsService.data.entity.TemplateEntityVersion;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

import java.util.ArrayList;
import java.util.List;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Schema(name = "TemplateDto", description = "Модель")
public class TemplateDto {
    @NotBlank
    @Schema(name ="templateId", description = "Код шаблона", defaultValue = "123")
    private Long templateId;
    @NotBlank
    @Schema(name ="templateFileId", description = "Код файла шаблона", defaultValue = "123")
    private Long templateFileId;
    @NotBlank
    @Schema(name ="headerFileId", description = "Код шапки шаблона", defaultValue = "123")
    private Long headerFileId;

    @NotBlank
    @Schema(name ="code", description = "Код шаблона", defaultValue = "template_1")
    private String code;
    @NotBlank
    @Schema(name ="name", description = "Наименование шаблона",defaultValue = "Шаблон 1")
    private String name;
    @NotBlank
    @Schema(name = "type", description = "Тип фаила")
    private String type;
    @NotBlank
    @Schema(name ="status", description = "Флаг активности", defaultValue = "1")
    private Boolean status;
    @NotBlank
    @Schema(name ="project", description = "Проект")
    private String project;
    @NotBlank
    @Schema(name ="version", description = "Версия активности")
    private Integer version;
    private TemplateFileDataDto templateFile;
    private TemplateFileDataDto headerFile;


    public static TemplateDto toDtoShort(TemplateEntity te) {
       return TemplateDto.builder()
                .code(te.getCode())
                .status(te.getStatus())
                .templateFile(new TemplateFileDataDto(te.getTemplate().getId(),te.getTemplate().getName(), te.getTemplate().getData()))
                .headerFile(te.getTemplateHeader()!=null?new TemplateFileDataDto(te.getTemplateHeader().getId(),te.getTemplateHeader().getName(), te.getTemplateHeader().getData()):null)
                .type(te.getType().getCode())
                .project(te.getProject())
                .version(te.getVersion())
                .build();
    }

    public static TemplateDto toDto(TemplateEntity te) {
        return TemplateDto.builder()
                .templateId(te.getId())
                .name(te.getName())
                .code(te.getCode())
                .status(te.getStatus())
                .project(te.getProject())
                .templateFile(new TemplateFileDataDto(te.getTemplate().getId(),te.getTemplate().getName(), te.getTemplate().getData()))
                .headerFile(te.getTemplateHeader()!=null?new TemplateFileDataDto(te.getTemplateHeader().getId(),te.getTemplateHeader().getName(), te.getTemplateHeader().getData()):null)
                .type(te.getType().getCode())
                .version(te.getVersion())
                .build();
    }

    public static TemplateDto toDtoAll(TemplateEntity te) {
        return TemplateDto.builder()
                .templateId(te.getId())
                .name(te.getName())
                .code(te.getCode())
                .status(te.getStatus())
                .project(te.getProject())
                .type(te.getType().getCode())
                .version(te.getVersion())
                .build();
    }

    public static Page<TemplateDto> toDtoList(Page<TemplateEntity> all){
        if(all.getContent()==null)return null;
        List<TemplateDto> dtos = new ArrayList<>();
        for (TemplateEntity te:all.getContent()) {
            dtos.add(TemplateDto.toDtoAll(te));
        }
        return new PageImpl<>(dtos, PageRequest.of(all.getNumber(),all.getSize()),all.getTotalElements());
    }

    public static Page<TemplateDto> toDtoList(Page<TemplateEntity> all, Page<TemplateEntityVersion> versions){
        if(all.getContent() == null && versions.getContent() == null) return null;

        List<TemplateDto> dtos = new ArrayList<>();
        if (all.getContent() != null){
            for (TemplateEntity te:all.getContent()) {
                dtos.add(TemplateDto.toDtoAll(te));
            }
        }
        if (versions.getContent() != null){
            for (TemplateEntityVersion ver:versions.getContent()) {
                dtos.add(TemplateDto.toDtoAll(new TemplateEntity(ver)));
            }
        }

        return new PageImpl<>(dtos, PageRequest.of(all.getNumber(),all.getSize()),all.getTotalElements());
    }

    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class TemplateFileDataDto{
        @NotBlank
        @Schema(name ="fileId", description = "Идентификатор файла", defaultValue = "4654654646")
        private Long fileId;
        @NotBlank
        @Schema(name ="fileName", description = "Название файла")
        private String fileName;
        @NotBlank
        @Schema(name ="fileData", description = "Название файла")
        private byte[] fileData;

    }

    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class GroupIndoDto{
        private String groupId;
        private String name;
    }
}


