package kz.project.printedFormsService.data.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Schema(name = "TemplateResponseDto", description = "Модель")
public class TemplateResponseDto {
    @NotBlank
    @Schema(name = "templateId", description = "Код шаблона", defaultValue = "123")
    private Long templateId;
    @NotBlank
    @Schema(name = "code", description = "Код шаблона", defaultValue = "template_1")
    private String code;
    @NotBlank
    @Schema(name = "name", description = "Наименование шаблона", defaultValue = "Шаблон 1")
    private String name;
    @NotBlank
    @Schema(name = "type", description = "Тип фаила")
    private String type;
    @NotBlank
    @Schema(name = "status", description = "Флаг активности", defaultValue = "1")
    private Boolean status;
    @NotBlank
    @Schema(name = "project", description = "Проект")
    private String project;
    @NotBlank
    @Schema(name = "version", description = "Версия активности")
    private Integer version;
    private TemplateDto.TemplateFileDataDto templateFile;
    private TemplateDto.TemplateFileDataDto headerFile;

    public static TemplateResponseDto toDto(TemplateEntity te) {
        return TemplateResponseDto.builder()
                .templateId(te.getId())
                .name(te.getName())
                .code(te.getCode())
                .status(te.getStatus())
                .project(te.getProject())
                .templateFile(new TemplateDto.TemplateFileDataDto(te.getTemplate().getId(), te.getTemplate().getName(), te.getTemplate().getData()))
                .headerFile(te.getTemplateHeader() != null ? new TemplateDto.TemplateFileDataDto(te.getTemplateHeader().getId(), te.getTemplateHeader().getName(), te.getTemplateHeader().getData()) : null)
                .type(te.getType().getCode())
                .version(te.getVersion())
                .build();
    }
}
