package kz.project.printedFormsService.data.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

import java.util.ArrayList;
import java.util.List;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Schema(name = "TemplateShortDto", description = "Модель")
public class TemplateShortDto {
    @NotBlank
    @Schema(name ="templateId", description = "Код шаблона", defaultValue = "123")
    private Long templateId;

    @NotBlank
    @Schema(name ="code", description = "Код шаблона", defaultValue = "template_1")
    private String code;
    @NotBlank
    @Schema(name ="name", description = "Наименование шаблона",defaultValue = "Шаблон 1")
    private String name;
    @NotBlank
    @Schema(name = "type", description = "Тип фаила")
    private String type;
    @NotBlank
    @Schema(name ="status", description = "Флаг активности", defaultValue = "1")
    private Boolean status;
    @NotBlank
    @Schema(name ="project", description = "Проект")
    private String project;
    @NotBlank
    @Schema(name ="version", description = "Версия активности")
    private Integer version;

    public static TemplateShortDto toDto(TemplateEntity te) {
        return TemplateShortDto.builder()
                .templateId(te.getId())
                .name(te.getName())
                .code(te.getCode())
                .status(te.getStatus())
                .project(te.getProject())
                .type(te.getType().getCode())
                .version(te.getVersion())
                .build();
    }

    public static Page<TemplateShortDto> toDtoList(Page<TemplateEntity> all) {
        if(all.getContent()==null)return null;
        List<TemplateShortDto> dtos = new ArrayList<>();
        for (TemplateEntity te:all.getContent()) {
            dtos.add(TemplateShortDto.toDto(te));
        }
        return new PageImpl<>(dtos, PageRequest.of(all.getNumber(),all.getSize()),all.getTotalElements());
    }
}
