package kz.project.printedFormsService.data.entity;

import jakarta.persistence.*;
import kz.project.printedFormsService.data.entity.dict.DTemplateType;

@Entity
@Table(name = "template", schema = "template_schema")
public class TemplateEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    private String code;
    private String name;
    private Boolean status;
    private String project;
    private Integer version;
    @ManyToOne
    private DTemplateType type;
    @OneToOne
    private TemplateFileInfoEntity template;
    @OneToOne
    private TemplateFileInfoEntity templateHeader;

    public TemplateEntity() {}
    public TemplateEntity(TemplateEntityVersion version){
        this.id = version.getId();
        this.code = version.getCode();
        this.name = version.getName();
        this.status = version.getStatus();
        this.project = version.getProject();
        this.version = version.getVersion();
        this.type = version.getType();
        this.template = version.getTemplate();
        this.templateHeader = version.getTemplateHeader();
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Boolean getStatus() {
        return status;
    }

    public void setStatus(Boolean status) {
        this.status = status;
    }

    public String getProject() {
        return project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public DTemplateType getType() {
        return type;
    }

    public void setType(DTemplateType type) {
        this.type = type;
    }

    public TemplateFileInfoEntity getTemplate() {
        return template;
    }

    public void setTemplate(TemplateFileInfoEntity template) {
        this.template = template;
    }

    public TemplateFileInfoEntity getTemplateHeader() {
        return templateHeader;
    }

    public void setTemplateHeader(TemplateFileInfoEntity templateHeader) {
        this.templateHeader = templateHeader;
    }

    @Override
    public boolean equals(Object o){
        if (this == o) return true;

        if (o == null || !(o instanceof TemplateEntity)) return false;

        TemplateEntity other = (TemplateEntity) o;

        /* Примитивы */
        boolean isCodeEqual = this.code == other.getCode() || (this.code != null && this.code.equals(other.getCode()));
        boolean isNameEqual = this.name == other.getName() || (this.name != null && this.name.equals(other.getName()));
        boolean isStatusEqual = this.status == other.getStatus() || (this.status != null && this.status.equals(other.getStatus()));
        boolean isProjectEqual = this.project == other.getProject() || (this.project != null && this.project.equals(other.getProject()));

        /* Сложные типы */
        boolean isTypeEqual = this.type == other.getType() || (this.type != null && this.type.equals(other.getType()));
        boolean isTemplateEqual = this.template == other.getTemplate() || (this.template != null && this.template.equals(other.getTemplate()));
        boolean isHeaderEqual = this.templateHeader == other.getTemplateHeader() || (this.templateHeader != null && this.templateHeader.equals(other.getTemplateHeader()));

        return isCodeEqual && isNameEqual && isStatusEqual && isProjectEqual && isTypeEqual && isTemplateEqual && isHeaderEqual;
    }

    @Override
    public int hashCode(){
        int hash = 17;

        if (code != null) hash = 31 * hash * code.hashCode();
        if (name != null) hash = 31 * hash * name.hashCode();
        if (status != null) hash = 31 * hash * status.hashCode();
        if (project != null) hash = 31 * hash * project.hashCode();

        if (type != null) hash = 31 * hash * type.hashCode();
        if (template != null) hash = 31 * hash * template.hashCode();
        if (templateHeader != null) hash = 31 * hash * templateHeader.hashCode();

        return hash;
    }
}
