package kz.project.printedFormsService.data.repository;

import kz.project.printedFormsService.data.entity.dict.DocumentCountByTemplate;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.tuple.Pair;
import org.jetbrains.annotations.NotNull;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Repository
@RequiredArgsConstructor
public class DocumentJdbcRepository {

    private final NamedParameterJdbcTemplate jdbcTemplate;

    public Map<LocalDate, Integer> getDocumentCountsByDay(@NotNull LocalDate startDate,
                                                          @NotNull LocalDate endDate,
                                                          @NotNull Set<String> projects) {
        return jdbcTemplate.query(
                        """
                                SELECT
                                    DATE_TRUNC('day', created_at) AS date,
                                    COUNT(1) as count
                                FROM
                                    document
                                WHERE
                                    DATE_TRUNC('day',created_at) >= :startDate
                                    AND  DATE_TRUNC('day',created_at) <= :endDate
                                    --AND project in(:projects)
                                GROUP BY
                                    date
                                ORDER BY
                                    date
                                """,
                        new MapSqlParameterSource(Map.of(
                                "startDate", startDate,
                                "endDate", endDate,
                                "projects", projects)
                        ),
                        (rs, i) -> Pair.of(rs.getDate("date").toLocalDate(), rs.getInt("count")))
                .stream()
                .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
    }


    public List<DocumentCountByTemplate> getDocumentsCountByTemplate(@NotNull LocalDate startDate,
                                                                     @NotNull LocalDate endDate,
                                                                     @NotNull Set<String> projects) {
        return jdbcTemplate.query(
                """
                        SELECT
                            d.template_id,
                            t.name,
                            COUNT(1) as count
                        FROM
                            template_schema.document d
                        inner join template_schema.template t on t.template_id = d.template_id
                        WHERE
                            DATE_TRUNC('day',created_at) >= :startDate
                          AND DATE_TRUNC('day',created_at) <= :endDate
                          --AND d.project in(:projects)
                        GROUP BY
                            d.template_id, t.name
                        ORDER BY
                            template_id      
                                        """,
                new MapSqlParameterSource(Map.of(
                        "startDate", startDate,
                        "endDate", endDate,
                        "projects", projects)
                ),
                (rs, i) -> DocumentCountByTemplate.builder()
                        .templateId(String.valueOf(rs.getInt("template_id")))
                        .templateName(rs.getString("name"))
                        .count(rs.getInt("count"))
                        .build());
    }
}
