package kz.project.printedFormsService.logging;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.format.datetime.DateFormatter;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.util.Locale;

@Configuration
public class LoggingConfig {
    private static final String DEFAULT_LOCALE = "ru_RU";
    private static final String DEFAULT_DATE_FORMAT = "dd:MM:yyyy hh:mm:ss";
    @Value("${process-logger.date-format}")
    private String dateFormatString;
    @Value("${process-logger.locale}")
    private String localeString;

    @Bean
    public DateFormatter dateFormatter(){
        return new DateFormatter(dateFormatString != null ? dateFormatString : DEFAULT_DATE_FORMAT);
    }

    @Bean
    public Locale locale(){
        return parseLocale(localeString != null ? localeString : DEFAULT_LOCALE);
    }

    @Bean
    public ZoneId zoneId(){
        return ZoneId.systemDefault();
    }

    private Locale parseLocale(String localeString){
        String[] localeParts = localeString.split("_");
        if (localeParts.length == 1) localeParts[1] = "";
        return new Locale(localeParts[0], localeParts[1]);
    }
}
