package kz.project.printedFormsService.service.impl;

import kz.project.printedFormsService.data.dto.DocumentByDateDto;
import kz.project.printedFormsService.data.dto.DocumentByDateDto.DocumentCount;
import kz.project.printedFormsService.data.dto.DocumentByTemplateDto;
import kz.project.printedFormsService.data.repository.DocumentJdbcRepository;
import kz.project.printedFormsService.service.DocumentsService;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.Map;
import java.util.Set;

@Service
@RequiredArgsConstructor
public class DocumentsServiceImpl implements DocumentsService {

    private final DocumentJdbcRepository documentJdbcRepository;

    @Override
    public @NotNull DocumentByDateDto getDocumentCountsByDate(@NotNull LocalDate startDate, @NotNull LocalDate endDate, @NotNull Set<String> projects) {
        Map<LocalDate, Integer> documentsByDate = documentJdbcRepository.getDocumentCountsByDay(startDate, endDate, projects);
        return new DocumentByDateDto("success", "Данные успешно получены")
                .setData(DocumentByDateDto.DocumentsData.builder()
                        .startDate(startDate)
                        .endDate(endDate)
                        .documentCounts(
                                documentsByDate.entrySet()
                                        .stream()
                                        .map(entry -> DocumentCount.builder()
                                                .date(entry.getKey())
                                                .count(entry.getValue())
                                                .build()

                                        )
                                        .toList()
                        )
                        .build()
                );
    }

    @Override
    public @NotNull DocumentByTemplateDto getDocumentCountsByTemplate(@NotNull LocalDate startDate, @NotNull LocalDate endDate, @NotNull Set<String> projects) {
        return new DocumentByTemplateDto("success", "Данные успешно получены")
                .setData(DocumentByTemplateDto.DocumentsData.builder()
                        .startDate(startDate)
                        .endDate(endDate)
                        .templateCounts(documentJdbcRepository.getDocumentsCountByTemplate(startDate, endDate, projects)
                                .stream()
                                .map(documentCountByTemplate -> DocumentByTemplateDto.TemplateCount.builder()
                                        .templateId(documentCountByTemplate.getTemplateId())
                                        .templateName(documentCountByTemplate.getTemplateName())
                                        .totalCount(documentCountByTemplate.getCount())
                                        .build()
                                )
                                .toList()
                        )
                        .build()
                );
    }
}
