package kz.project.printedFormsService.service.impl;

import kz.project.printedFormsService.data.dto.GitlabMultipartFile;
import kz.project.printedFormsService.service.GitlabClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.io.File;

@Service
public class GitlabClientImpl implements GitlabClient {

    private final WebClient webClient;

    public GitlabClientImpl(@Value("${gitlab.base-url}") String baseUrl, @Value("${gitlab.token}") String token) {
        this.webClient = WebClient.builder()
                .baseUrl(baseUrl)
                .defaultHeader("PRIVATE-TOKEN", token)
                .build();
    }

    //http://gitlab.lan.arta.kz/d.ermakov/templates/-/raw/master/dto.txt?inline=false
    @Override
    public Mono<byte[]> downloadFileFromGitLab(String projectName, String filePath) {
        String urlPath = String.format("/api/v4/projects/%s/repository/files/%s/raw", projectName, filePath);
        return webClient.get()
                .uri(uriBuilder -> uriBuilder
                        .path(urlPath)
                        .queryParam("ref", "master") // or the branch you want to fetch from
                        .build())
                .retrieve()
                .bodyToMono(byte[].class)
                .doOnError(error -> System.err.println("Error downloading file: " + error.getMessage()));
    }

    @Override
    public Mono<String> downloadFileContentAsString(String projectName, String filePath) {
        String urlPath = String.format("/api/v4/projects/%s/repository/files/%s/raw", projectName, filePath);
        return webClient.get()
                .uri(uriBuilder -> uriBuilder
                        .path(urlPath)
                        .queryParam("ref", "master") // or the branch you want to fetch from
                        .build())
                .retrieve()
                .bodyToMono(String.class)
                .doOnError(error -> System.err.println("Error downloading file: " + error.getMessage()));
    }

    @Override
    public MultipartFile downloadFileAsMultipartFile(String projectName, String filePath) {
        String urlPath = String.format("/api/v4/projects/%s/repository/files/%s/raw", projectName, filePath);
        return webClient.get()
                .uri(uriBuilder -> uriBuilder
                        .path(urlPath)
                        .queryParam("ref", "master") // or the branch you want to fetch from
                        .build())
                .retrieve()
                .bodyToMono(byte[].class)
                .map(bytes -> {
                    File file = new File(filePath);
                    return new GitlabMultipartFile(file.getName(), file.getName(), null, bytes);
                })
                .doOnError(error -> System.err.println("Error downloading file: " + error.getMessage()))
                .block();
    }
}
